(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2009-2013 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: segmentation_spec.mli,v 1.75 2013-10-31 15:19:38 fclement Exp $ *)

open Segmentation_types;;
open Configuration_types;;
open Display_types;;

(* Scalar. *)

module type SCALAR =
sig
  include Ordered_field_spec.ORDERED_FIELD;;
  
  (*val digest : t Utype_digest.t;;*)
  
  val cut : t cutting;;
  val mean : t gimage -> t;;
  val cost_sub : t gimage -> t gimage -> t cost;;
  val cost : (t, 'a) pimage -> t cost;;
  val add_cost_decrease :
    t indicator ->
    t zc_gimage -> t gimage -> Umatrix_nonempty_set.dichotomy ->
    t indicator
  ;;
end
;;

module type SCALAR_MAKER =
  functor (Of : Ordered_field_spec.ORDERED_FIELD) ->
  SCALAR
;;

(* Codomain. *)

module type CODOMAIN_STRUCTURE =
sig
  val get_component_of_color : color -> Unatural.t -> int;;
end
;;

module type CODOMAIN =
sig
  include Tuple_spec.TUPLE;;
  include CODOMAIN_STRUCTURE;;
  
  (*val digest : ('a, 'a t) Utype_digest.lifter;;*)
  
  val all_channels : component_number t;;
  val complementary : 'a t -> 'a list t;;
  
  val of_color : (int -> 'a) -> color -> 'a t;;
  val to_color : ('a -> int) -> 'a t -> color
  ;;
end
;;

module type CODOMAIN_MAKER =
  functor (T : Tuple_spec.TUPLE) ->
  functor (Cs : CODOMAIN_STRUCTURE) ->
  CODOMAIN
;;

module type CODOMAIN_OF_VECTOR =
  functor (V : Vector_spec.VECTOR) ->
  CODOMAIN with type 'a t = 'a V.t
;;

(* State. *)

module type STATE =
sig
  type scalar;;
  type 'a codomain;;
  
  type t = State of (scalar, other_components) zonation codomain
  and other_components = component_number list * t
  ;;
  
  (*val digest : t Utype_digest.t;;*)
  
  val same : t -> t -> bool;;
  val print : t Format_printer.printer;;
  
  val gimage_components : scalar codomain gimage -> scalar gimage codomain;;
  val gimage_combine : scalar gimage codomain -> scalar codomain gimage;;
  
  val gimage_datas : t -> scalar gimage codomain;;
  val cimages : t -> (scalar, other_components) cimage codomain;;
  val cost : t -> scalar cost;;
  
  val init : unit -> t;;
  val final : Ufilename.t -> t -> unit;;
end
;;

module type STATE_MAKER =
  functor (S : SCALAR) ->
  functor (C : CODOMAIN) ->
  STATE with type scalar = S.t and type 'a codomain = 'a C.t
;;

(* Display. *)

module type DISPLAY_QUANTITY =
sig
  type scalar;;
  type other_components;;
  
  val setup :
    Unatural.t ->
    graphics_kind -> min_window_size -> max_window_size -> image_size -> unit
  ;;
  
  val set_quantity_and_draw :
    quantity -> (scalar, other_components) zonation Tuple.Triple.t -> unit
  ;;
  val update_quantity_and_draw :
    quantity -> (scalar, other_components) zonation Tuple.Triple.t -> unit
  ;;
end
;;

module type DISPLAY_QUANTITY_MAKER =
  functor (S : SCALAR) ->
  functor (St : STATE with type scalar = S.t) ->
  DISPLAY_QUANTITY with
    type scalar = S.t and
    type other_components = St.other_components
;;

module type DISPLAY =
sig
  type history;;
  
  val init : history -> unit;;
  val display : history -> unit;;
end
;;

module type DISPLAY_MAKER =
  functor (S : SCALAR) ->
  functor (C : CODOMAIN) ->
  functor (St : STATE with type scalar = S.t and type 'a codomain = 'a C.t) ->
  functor (H : History_spec.HISTORY with type state = St.t) ->
  DISPLAY with type history = H.t
;;

(* Segmentation. *)

module type SEGMENTATION =
sig
  type scalar;;
  type 'a codomain;;
  
  val algorithm : unit -> unit;;
end
;;

module type SEGMENTATION_MAKER =
  functor (S : SCALAR) ->
  functor (C : CODOMAIN) ->
  SEGMENTATION with type scalar = S.t and type 'a codomain = 'a C.t
;;
