(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uarray1_with_fortran_layout.mli,v 1.11 2014/01/10 15:28:04 fclement Exp $ *)

(** {3 More operations on one-dimensional bigarrays with Fortran layout} *)

open Bigarray;;

val init :
    ('a, 'b) kind -> int -> (int -> 'a) -> ('a, 'b, fortran_layout) Array1.t;;
(** [init kind dim f] returns a fresh
    one-dimensional bigarray of size [dim] whose element number [i] is of kind
    [kind] and is initialized to [f i]. *)

val make : ('a, 'b) kind -> int -> 'a -> ('a, 'b, fortran_layout) Array1.t;;
(** [make kind dim value] returns a fresh
    one-dimensional bigarray of size [dim] whose elements are of kind  [kind]
    and are initialized to [value]. *)

(** {6 Iterators} *)

val iter : ('a -> unit) -> ('a, 'b, fortran_layout) Array1.t -> unit;;
(** [iter f a] applies function [f] in turn to all
    the elements of the bigarray [a]. *)

val map :
    ('a -> 'a) -> ('a, 'b, fortran_layout) Array1.t ->
      ('a, 'b, fortran_layout) Array1.t;;
(** [map f a] applies function [f] to all the
    elements of the bigarray [a] and returns a fresh bigarray with the results
    given by [f]. *)

val fold_left :
    ('a -> 'b -> 'a) -> 'a -> ('b, 'c, fortran_layout) Array1.t -> 'a;;
(** [fold_left f a ] computes 
   [f (... (f (f x a.{1}) a.{2}) ...) a.{n}],
   where [n] is the length of the bigarray [a]. *)

(** {6 Iterators on two bigarrays} *)

val fold_left2 :
    ('a -> 'b -> 'c -> 'a) -> 'a -> ('b, 'd, fortran_layout) Array1.t ->
      ('c, 'e, fortran_layout) Array1.t -> 'a;;
(** [fold_left2 f a b] computes 
   [f (... (f (f x a.{1} b.{1}) a.{2} b.{2}) ...) a.{n} b.{n}],
   where [n] is the length of the bigarrays [a] and [b].
   
   Raise [Invalid_argument] if the two bigarrays have different lengths. *)
