(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2004-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: usubarray.ml,v 1.5 2014/01/10 15:28:04 fclement Exp $ *)

(* Subarray operations. *)

open Usubarray_types;;

let fold_left f accu = function
  | { support = a; indices = Range { offset = ofs; length = len; }; } ->
      let rec loop accu idx =
        if idx = len then accu else
        let x = Array.unsafe_get a (Unatural.to_int idx) in
        let accu = f accu x in
        loop accu (Unatural.succ idx) in
      loop accu ofs
  | { support = a; indices = Indices ia; } ->
      Array.fold_left
        (fun accu idx ->
           let x = Array.unsafe_get a (Unatural.to_int idx) in
           f accu x)
        accu ia
;;

let fold_left_i f accu = function
  | { support = a; indices = Range { offset = ofs; length = len; }; } ->
      let rec loop accu i idx =
        if i = len then accu else
        let x = Array.unsafe_get a (Unatural.to_int idx) in
        let accu = f accu i x in
        loop accu (Unatural.succ i) (Unatural.succ idx) in
      loop accu Unatural._0_ ofs
  | { support = a; indices = Indices ia; } ->
      Uarray.fold_left_i
        (fun accu i idx ->
           let x = Array.unsafe_get a (Unatural.to_int idx) in
           f accu (Unatural.of_int i) x)
        accu ia
;;

let find p = function
  | { support = a; indices = Range { offset = ofs; length = len; }; } ->
      let rec loop idx =
        if idx >= len then raise Not_found else
        let x = Array.unsafe_get a (Unatural.to_int idx) in
        if p x then x else loop (Unatural.succ idx) in
      loop ofs
  | { support = a; indices = Indices ia; } ->
      let idx =
        Uarray.find
          (fun idx ->
             let x = Array.unsafe_get a (Unatural.to_int idx) in
             p x)
          ia in
      Array.unsafe_get a (Unatural.to_int idx)
;;

let find_all p = function
  | { support = a; indices = Range { offset = ofs; length = len; }; } ->
      let rec loop accu i =
        if i >= len then accu else
        let x = Array.unsafe_get a (Unatural.to_int i) in
        let accu = if p x then x :: accu else accu in
        loop accu (Unatural.succ i) in
      Ulist.rev (loop [] ofs)
  | { support = a; indices = Indices ia; } ->
      let idxs =
        Uarray.find_all
          (fun i -> p (Array.unsafe_get a (Unatural.to_int i))) ia in
      Ulist.map (fun idx -> Array.unsafe_get a (Unatural.to_int idx)) idxs
;;
